#include <TMath.h>
#include <TH1D.h>
#include <TProfile.h>
#include <TFile.h>

Double_t GetRes(Double_t _chi)
{
  Double_t con = TMath::Sqrt(TMath::Pi() / 2) / 2;
  Double_t arg = _chi * _chi / 4.;
  Double_t res = con * _chi * exp(-arg) * (TMath::BesselI0(arg) + TMath::BesselI1(arg));
  return res;
}
	     
Double_t GetChi(Double_t _res, Int_t accuracy = 50)
{
  Double_t chi = 2.0;
  Double_t delta = 1.0;
  for (int i = 0; i < accuracy; i++)
  {
    if (GetRes(chi) < _res)
    {
      chi = chi + delta;
    }
    else
    {
      chi = chi - delta;
    }
      delta = delta / 2.;
    }
    return chi;
}

void CalcRes(const Char_t *resFileName = "./oResolutionTest.root", const Char_t *resFitName = "./oResFitTest.root")
{

  //const std::map<int,int> CentBins [] = {{1,75},{2,65},{3,55},{4,45},{5,35},{6,25},{7,15},{8,7.5},{9,2.5}};
  const int NGaps = 4;
  TProfile *pRes2Sqr[NGaps];
  TProfile *pRes3Sqr[NGaps];

  TH1D *hRes2Half[NGaps];
  TH1D *hRes3Half[NGaps];

  TH1D *hRes2FULL[NGaps];
  TH1D *hRes3FULL[NGaps];

  TH1D *hRes2Chi[NGaps];
  TH1D *hRes3Chi[NGaps];

  TFile *fi = new TFile(resFileName,"read");
  fi->cd();

  for (int i=0; i<NGaps; i++)
  {
    pRes2Sqr[i] = (TProfile*) fi->Get(Form("p_res2_EP_gap%i",i));
    pRes3Sqr[i] = (TProfile*) fi->Get(Form("p_res3_EP_gap%i",i));

    hRes2Half[i] = new TH1D(Form("hRes2Half%i",i),Form("hRes2Half%i",i),9,-0.5,8.5);
    hRes3Half[i] = new TH1D(Form("hRes3Half%i",i),Form("hRes3Half%i",i),9,-0.5,8.5);

    hRes2FULL[i] = new TH1D(Form("hRes2FULL%i",i),Form("hRes2FULL%i",i),9,-0.5,8.5);
    hRes3FULL[i] = new TH1D(Form("hRes3FULL%i",i),Form("hRes3FULL%i",i),9,-0.5,8.5);

    hRes2Chi[i] = new TH1D(Form("hRes2Chi%i",i),Form("hRes2Chi%i",i),9,-0.5,8.5);
    hRes3Chi[i] = new TH1D(Form("hRes3Chi%i",i),Form("hRes3Chi%i",i),9,-0.5,8.5);
  }

  int bin = 0;
  
  //Fill ResHalf
  for (int i=0; i<NGaps; i++)
  {
    for (int ibin=1; ibin<=pRes2Sqr[i]->GetNbinsX(); ibin++)
    {
      hRes2Half[i]->SetBinContent(ibin,TMath::Sqrt(pRes2Sqr[i]->GetBinContent(ibin)));
      hRes2Half[i]->SetBinError(ibin,(Double_t) (0.5*pRes2Sqr[i]->GetBinError(ibin) / TMath::Sqrt(pRes2Sqr[i]->GetBinContent(ibin))) );

      hRes3Half[i]->SetBinContent(ibin,TMath::Sqrt(pRes3Sqr[i]->GetBinContent(ibin)));
      hRes3Half[i]->SetBinError(ibin,(Double_t) (0.5*pRes3Sqr[i]->GetBinError(ibin) / TMath::Sqrt(pRes3Sqr[i]->GetBinContent(ibin))) );
    }
  }

  Double_t chi = 0.;

  //Fill Chi & FULL
  for (int i=0; i<NGaps; i++)
  {
    for (int ibin=1; ibin<=pRes2Sqr[i]->GetNbinsX(); ibin++)
    {
      chi = GetChi(hRes2Half[i]->GetBinContent(ibin));
      hRes2Chi[i]->SetBinContent(ibin,GetRes(chi));
      hRes2Chi[i]->SetBinError(ibin,hRes2Half[i]->GetBinError(ibin));
      hRes2FULL[i]->SetBinContent(ibin,GetRes(TMath::Sqrt(2)*chi));
      hRes2FULL[i]->SetBinError(ibin,TMath::Sqrt(2)*hRes2Half[i]->GetBinError(ibin));

      chi = GetChi(hRes3Half[i]->GetBinContent(ibin));
      hRes3Chi[i]->SetBinContent(ibin,GetRes(chi));
      hRes3Chi[i]->SetBinError(ibin,hRes3Half[i]->GetBinError(ibin));
      hRes3FULL[i]->SetBinContent(ibin,GetRes(TMath::Sqrt(2)*chi));
      hRes3FULL[i]->SetBinError(ibin,TMath::Sqrt(2)*hRes3Half[i]->GetBinError(ibin));
    }
  }

  TFile *fo = new TFile(resFitName, "recreate");
  fo->cd();

  for (int i=0; i<NGaps; i++)
  {
    hRes2Half[i]->Write();
    hRes2Chi[i]->Write();
    hRes2FULL[i]->Write();
  }

  for (int i=0; i<NGaps; i++)
  {
    hRes3Half[i]->Write();
    hRes3Chi[i]->Write();
    hRes3FULL[i]->Write();
  }

}
